/*
 * Copyright (c) 2020 Arm Limited. All rights reserved.
 *
 * This Software is licensed under an Arm proprietary license.
 *
 *      Name:    main.c
 *      Purpose: Example integrating X-CUBE-STL in FuSa RTS project
 * -----------------------------------------------------------------------------
 */

#include "RTE_Components.h"      /* Component selection */
#include CMSIS_device_header
#include "cmsis_os2.h"
#include "stdlib.h"
#include "stl_user_api.h"

#define STL_THREAD_STK_SZ (512U)
static uint64_t stl_thread_stk[STL_THREAD_STK_SZ / 8];
static const osThreadAttr_t stl_thread_attr = {
  .stack_mem  = &stl_thread_stk[0],
  .stack_size = sizeof(stl_thread_stk),
	.priority = osPriorityHigh
};

/* Array for CPU ARM Core Test Module test list */
static STL_TmEnable_t aCpuTmEnable[STL_CPU_TM_MAX]={
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 1 */
  STL_TEST_DISABLE,/* CPU ARM Core Test Module 1L */  // choose one test to run TM1 or TM1L
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 2 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 3 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 4 */	
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 5 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 6 */			
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 7 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 8 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 9 */
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 10 */	
  STL_TEST_ENABLE, /* CPU ARM Core Test Module 11 */		
}; 

static osThreadId_t tid_app_main;  /* Thread id of thread: app_main */
static osThreadId_t tid_thrLED;    /* Thread id of thread: LED */
static osThreadId_t tid_thrBUT;    /* Thread id of thread: BUT */
static osThreadId_t tid_thrSTL;    /* Thread id of thread: BUT */

static volatile uint32_t delay_val = 500U;

/* Private defines -----------------------------------------------------------*/
#define FLASH_SECTION_SIZE 1024U /* Fixed */
#define TEST_FLASH_SECTIONS 48  /* Customizable. Amount of flash sections to be tested in total. Corresponds to the code size*/
#define TEST_FLASH_SECTION_NB 1 /* Customizable. Amount of flash sections for single test run*/
#define TEST_ROM_START_ADDR 0x08000000U 
#define TEST_ROM_END_ADDR (TEST_ROM_START_ADDR + TEST_FLASH_SECTIONS*FLASH_SECTION_SIZE) 


#ifdef STL_DISABLE_RAM_BCKUP_BUF
#define RAM_BACKUP_BUFFER_SIZE 0
#else
#define RAM_BACKUP_BUFFER_SIZE 32 /* Fixed value as in stl_user_param.c */
#endif
#define RAM_SECTION_SIZE 128U  /* Fixed */
#define TEST_RAM_START_ADDR (0x20000000U + RAM_BACKUP_BUFFER_SIZE)
#define TEST_RAM_END_ADDR 0x20004000U /* Customizable, but can be full RAM */
#define TEST_RAM_SECTION_NB 1 /* Amount of RAM section tested at once*/

/* Functions Definition ------------------------------------------------------*/
/* STL tests in SVC context */
extern void svc_StlCpuTMTest(uint32_t testNumber);
extern void svc_StlFlashTest(uint32_t startAddr, uint32_t endAddr, uint32_t sections);
extern void svc_StlRamTest(uint32_t startAddr, uint32_t endAddr, uint32_t sections);

/* Error handling function */
extern void stop_on_error(uint32_t cond);

/*----------------------------------------------------------------------------
  thrSTL: run STL
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrSTL(void *argument) {
  
	(void)argument;
	
  uint32_t i = 0;
	uint32_t currRomStartAddr = 0;
	uint32_t currRomEndAddr   = 0;
	uint32_t RomSections      = TEST_FLASH_SECTION_NB;
  
	for (;;) {
		
    /** Run CPU tests *******/
		for (i=0; i<STL_CPU_TM_MAX;i++) {
		  if(aCpuTmEnable[i]==STL_TEST_ENABLE) {
				osDelay(1);
				svc_StlCpuTMTest(i);
			}
		}
		
		/** Run Flash tests *******/
		currRomStartAddr = TEST_ROM_START_ADDR;
		while (currRomStartAddr <= TEST_ROM_END_ADDR-(RomSections*FLASH_SECTION_SIZE)) {
		  currRomEndAddr = currRomStartAddr+RomSections*FLASH_SECTION_SIZE - 1;
		  osDelay(1);
		  svc_StlFlashTest(currRomStartAddr,currRomEndAddr,RomSections);
			currRomStartAddr = currRomEndAddr + 1;
		}
		
		/** Run RAM tests *******/
		uint32_t currRamStartAddr = TEST_RAM_START_ADDR;
	  uint32_t currRamEndAddr   = 0;
	  uint32_t RamSections      = TEST_RAM_SECTION_NB;

		while (currRamStartAddr<=TEST_RAM_END_ADDR-RamSections*RAM_SECTION_SIZE) {
			currRamEndAddr = currRamStartAddr+RamSections*RAM_SECTION_SIZE-1;
		  osDelay(1);
		  svc_StlRamTest(currRamStartAddr,currRamEndAddr,RamSections);
			
			currRamStartAddr = currRamEndAddr+1;
		}
		
		/** Delay before re-running the tests *******/
		osDelay(5000);
	}
}

/*----------------------------------------------------------------------------
  thrLED: blink LED
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrLED(void *argument) {
  (void)argument;
	
	uint32_t led_max    = 0U;//LED_GetCount();
  uint32_t led_num    = 0U;

  for (;;) {
    osThreadFlagsWait(0x0001U, osFlagsWaitAny ,osWaitForever);
    //LED_On(led_num);                                           /* Turn specified LED on */
    osThreadFlagsWait(0x0001U, osFlagsWaitAny ,osWaitForever);
    //LED_Off(led_num);                                          /* Turn specified LED off */

    led_num++;                                                   /* Change LED number */
    if (led_num >= led_max) {
      led_num = 0U;                                              /* Restart with first LED */
    }
  }
}

/*----------------------------------------------------------------------------
  thrBUT: check button state
 *----------------------------------------------------------------------------*/
__NO_RETURN static void thrBUT(void *argument) {
//  uint32_t button_msk = 0(1U << Buttons_GetCount()) - 1U;

  (void)argument;

  for (;;) {
    osDelay(delay_val);                                        /* Wait */
    //while (Buttons_GetState() & (button_msk));               /* Wait while holding USER button */
    osThreadFlagsSet(tid_thrLED, 0x0001U);
  }
}
/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN static void app_main (void *argument) {

  (void)argument;

  //LED_Initialize();                                       /* initalize LEDs */
  //Buttons_Initialize();                                   /* initalize Buttons */

  tid_thrBUT = osThreadNew (thrBUT, NULL, NULL);            /* create virtual BUT thread */
  if (tid_thrBUT == NULL) { /* add error handling */ }

  tid_thrLED = osThreadNew (thrLED, NULL, NULL);            /* create virtual LED thread */
  if (tid_thrLED == NULL) { /* add error handling */ }

  tid_thrSTL = osThreadNew (thrSTL, NULL, &stl_thread_attr); /* create STL thread */
  if (tid_thrSTL == NULL) { /* add error handling */ }
	
	osThreadFlagsWait(0x0001U, osFlagsWaitAny ,osWaitForever);
  for (;;) {}
}

/*----------------------------------------------------------------------------
  main
 *----------------------------------------------------------------------------*/
int main(void) {
	
	volatile osStatus_t ret;
  SystemCoreClockUpdate(); 
	
  /* Initialize kernel */
  ret = osKernelInitialize();
  stop_on_error(ret != osOK);
	
  osKernelInitialize();   // Initialize FuSa RTX RTOS
	
  tid_app_main = osThreadNew(app_main, NULL, NULL); // Create application main thread
	stop_on_error(tid_app_main == NULL);
	
  /* Start thread execution */
  if (osKernelGetState() == osKernelReady) {
    ret = osKernelStart();
    stop_on_error(ret != osOK);
  }
 /* Infinite loop */
 while(1){}
}
