/*
 * Copyright (c) 2020 Arm Limited. All rights reserved.
 *
 * This Software is licensed under an Arm proprietary license.
 *      
 *      Name:    stl_run.c
 *      Purpose: Example module for executing X-CUBE-STL test
 *
 * -----------------------------------------------------------------------------
 */
#include "stdlib.h"
#include "stl_user_api.h"
#include "EventRecorder.h"
#include "cmsis_compiler.h"

/* Private typedef -----------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
// Array mapping CPU TM number to a corresponding STL API function 
static STL_Status_t (*stlCpuTests[STL_CPU_TM_MAX])(STL_TmStatus_t *) = {
	STL_SCH_RunCpuTM1,
	STL_SCH_RunCpuTM1L,
	STL_SCH_RunCpuTM2,
  STL_SCH_RunCpuTM3,
  STL_SCH_RunCpuTM4,
  STL_SCH_RunCpuTM5,		
  STL_SCH_RunCpuTM6,
  STL_SCH_RunCpuTM7,		
  STL_SCH_RunCpuTM8,
  STL_SCH_RunCpuTM9,
  STL_SCH_RunCpuTM10,
  STL_SCH_RunCpuTM11
};
/* Global variables ----------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
static void StlErrorHandler(void); 
/* Functions Definition ------------------------------------------------------*/
extern void stop_on_error(uint32_t cond);
extern void StlCpuTMTest(uint32_t testNumber);
extern void StlFlashTestRun(uint32_t startAddr, uint32_t endAddr, uint32_t sections);
extern void StlRamTestRun(uint32_t startAddr, uint32_t endAddr, uint32_t sections);

/*----------------------------------------------------------------------------
 * Function 'StlErrorHandler' to catch STL execution errors
 *---------------------------------------------------------------------------*/
static void StlErrorHandler(void) {
	/* User error handling */
	stop_on_error(1);
}

/*----------------------------------------------------------------------------
 * Function 'StlCpuTMTest' to execute STL tests as specified by the argument
 *---------------------------------------------------------------------------*/
void StlCpuTMTest(uint32_t testNumber) {
  STL_TmStatus_t StlCpuTmStatus = STL_ERROR;
  
	/* verify testNumber */
  if (testNumber > STL_CPU_TM_MAX) {
    StlErrorHandler();  
  }
  /* Init STL */
  else if (STL_SCH_Init() != STL_OK) {
    StlErrorHandler();      /* STL Defense Programming */
  }
  /* Call STL_SCH_RunCpuTM<testNumber> */
  else if (((*stlCpuTests[testNumber])(&StlCpuTmStatus)) != STL_OK) {
    StlErrorHandler();      /* STL Defense Programming */
  }
  else if (StlCpuTmStatus != STL_PASSED) {
    StlErrorHandler();      /* STL Tests FAILED  */
  }
}

/*----------------------------------------------------------------------------
 * Function 'StlFlashTestRun' to execute STL Flash test as specified by the arguments
 *---------------------------------------------------------------------------*/
void StlFlashTestRun(uint32_t startAddr, uint32_t endAddr, uint32_t sections) {	
  
	STL_MemSubset_t FlashSubset;
  STL_MemConfig_t FlashConfig;

  FlashSubset.StartAddr = startAddr;
  FlashSubset.EndAddr = endAddr;
  FlashSubset.pNext = NULL;
  FlashConfig.pSubset = &FlashSubset;
  FlashConfig.NumSectionsAtomic = sections;
  STL_TmStatus_t StlFlashStatus = STL_ERROR;  
	
	  /* Init STL */
  if (STL_SCH_Init() != STL_OK) {
    StlErrorHandler();  /* STL Defense Programming */
  }
	
  /* Init Flash TM */
  else if (STL_SCH_InitFlash(&StlFlashStatus) != STL_OK) {
    StlErrorHandler();  /* STL Defense Programming */
  }
  /* check Flash TM status is reset to STL_NOT_TESTED */
  else if (StlFlashStatus != STL_NOT_TESTED) {
    StlErrorHandler();  /* STL Defense Programming */
  }
	/* Configure Flash TM */
  if (STL_SCH_ConfigureFlash(&StlFlashStatus, &FlashConfig) == STL_KO) {
    StlErrorHandler(); /* STL Defense Programming */
  }
  /* Check Flash TM status is reset to STL_NOT_TESTED */
  else if (StlFlashStatus != STL_NOT_TESTED) {
    StlErrorHandler(); /* STL Defense Programming */
  }
  /* Run Flash TM */
  else if (STL_SCH_RunFlashTM(&StlFlashStatus) != STL_OK) {
    StlErrorHandler(); /* STL Defense Programming */
  }
  /* check Flash TM status */
  else if (StlFlashStatus != STL_PASSED) {
    StlErrorHandler(); /* STL Tests FAILED */
  }
}

/*----------------------------------------------------------------------------
 * Function 'StlRamTestRun' to execute STL RAM test as specified by the arguments
 *---------------------------------------------------------------------------*/
void StlRamTestRun(uint32_t startAddr, uint32_t endAddr, uint32_t sections) {	
	
	/* RAM configurations: */
  STL_MemSubset_t RamSubset;
  STL_MemConfig_t RamConfig;

  RamSubset.StartAddr = startAddr;
  RamSubset.EndAddr   = endAddr;
  RamSubset.pNext     = NULL;
  RamConfig.pSubset   = &RamSubset;
  RamConfig.NumSectionsAtomic = sections; 
	
  STL_TmStatus_t StlRamStatus = STL_ERROR;
  /* Init RAM TM */
  if (STL_SCH_InitRam(&StlRamStatus) != STL_OK) {
    StlErrorHandler();      /* STL Defense Programming */
  }
  /* check RAM TM status is reset to STL_NOT_TESTED */
  else if (StlRamStatus != STL_NOT_TESTED) {
    StlErrorHandler();       /* STL Defense Programming */
  }
  /* Configure RAM TM */
  else if (STL_SCH_ConfigureRam(&StlRamStatus, &RamConfig) != STL_OK) {
    StlErrorHandler();      /* STL Defense Programming */
  }
  /* check RAM TM status is reset to STL_NOT_TESTED */
  else if (StlRamStatus != STL_NOT_TESTED) {
    StlErrorHandler();       /* STL Defense Programming */
  }
  /* run RAM TM */
  else if (STL_SCH_RunRamTM(&StlRamStatus) != STL_OK) {
    StlErrorHandler();      /* STL Defense Programming */
  }
  /* check RAM TM status */
  else if (StlRamStatus != STL_PASSED) {
    StlErrorHandler();      /* STL Tests FAILED */
  }
}
